#!/usr/bin/env python
# -*-coding: utf-8 -*-

#-----------------------------------------------------------------------------------------------------------------------

# Name:		data_reduction.py
# Author:	Maximilian A. Beeskow
# Version:	v1.0.77
# Date:		16.05.2025

#-----------------------------------------------------------------------------------------------------------------------

## MODULES -------------------------------------------------------------------------------------------------------------
import pandas as pd
import numpy as np
import os, time

## CLASSES -------------------------------------------------------------------------------------------------------------
class DataExtraction:
    def __init__(self, filename_long=None):
        self.filename_long = filename_long

    def find_header(self):
        file_content = open(self.filename_long[0])
        found_delimiter = False
        common_header_elements = ["Time", "Li7", "Na23", "Mg24", "Si29", "S32", "Cl35", "K39", "Ca44"]
        set_common = set(common_header_elements)
        for index, line in enumerate(file_content):
            if len(line) > 100 and found_delimiter == False:
                var_delimiter = self.get_delimiter(var_str=line)
                index_header = index
                line_header = line
                possible_header_elements = line_header.split(var_delimiter)
                if "\n" in possible_header_elements[-1]:
                    possible_header_elements[-1] = possible_header_elements[-1].replace("\n", "")
                set_possible = set(possible_header_elements)
                set_intersection = list(set_common & set_possible)
                if len(set_intersection) > 0:
                    found_delimiter = True
            else:
                set_intersection = []
            if len(set_intersection) > 0:
                break

        return index_header, possible_header_elements, var_delimiter

    def get_delimiter(self, var_str):
        possible_delimiter = [",", ";", " "]
        n_str = len(var_str)
        condition = False
        while condition == False:
            for delimiter in possible_delimiter:
                for element in var_str.split(delimiter):
                    n_element = len(element)
                    if n_element != n_str:
                        delimiter_working = delimiter
                        condition = True
                    if condition == True:
                        break
                if condition == True:
                    break

        return delimiter_working

    def find_dataset_end(self, var_delimiter, var_index_header):
        file_content = open(self.filename_long[0])
        last_data_line = 0
        for index, line in enumerate(file_content):
            if index > var_index_header:
                line_elements = line.split(var_delimiter)
                if len(line_elements) > 0:
                    if len(line_elements) > 2:
                        last_data_line = index
                    else:
                        break
                else:
                    break

        return last_data_line

    def get_file_creation_time(self, var_delimiter, meta_data=False):
        if meta_data == False:
            file_content = open(self.filename_long[0])
            common_date_keys = [
                "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday", "Mon", "Tue", "Wed",
                "Thu", "Fri", "Sat", "Sun", "Acquired", "Printed", "Created", "January", "February", "March", "April",
                "May", "June", "July", "August", "September", "October", "November", "December"]
            set_common = set(common_date_keys)
            for index, line in enumerate(file_content):
                if len(line) < 100:
                    line_elements = line.split(var_delimiter)
                    if "\n" in line_elements[-1]:
                        line_elements[-1] = line_elements[-1].replace("\n", "")
                    set_possible = set(line_elements)
                    set_intersection = list(set_common & set_possible)
                    if len(set_intersection) > 0:
                        break
                    else:
                        for common_key in common_date_keys:
                            if common_key in line_elements:
                                print(common_key)
        else:
            path = self.filename_long[0]
            time_creation = os.stat(path).st_birthtime
            time_creation_converted = time.ctime(time_creation)
            t_obj = time.strptime(time_creation_converted)
            t_stamp = time.strftime("%H:%M:%S", t_obj)

        return line_elements

    def get_measurements(self, delimiter, skip_header, skip_footer):
        self.delimiter = delimiter
        self.skip_header = skip_header
        self.skip_footer = skip_footer

        dataframe = pd.read_csv(
            self.filename_long, sep=self.delimiter, header=self.skip_header, skipfooter=self.skip_footer,
            engine="python", encoding="latin1")
        dataframe_blank = dataframe.loc[dataframe.isnull().all(1)]

        if len(dataframe_blank) > 0:
            first_blank_index = dataframe_blank.index[0]
            dataframe = dataframe[:first_blank_index]
        var_columns = dataframe.columns

        for column in var_columns:
            dataframe[column] = dataframe[column].astype(float)

        return dataframe

    def get_isotopes(self, dataframe):
        list_names = list(dataframe.columns.values)
        list_names.pop(0)
        list_isotopes = list_names

        return list_isotopes

    def get_times(self, dataframe):
        df_times = dataframe.iloc[:, 0]

        return df_times

class IntensityQuantification:
    def __init__(self, dataframe, internal_standard=None, mode="specific", project_type="MA", results_container=None):
        self.dataframe = dataframe
        self.internal_standard = internal_standard
        self.mode = mode
        self.project_type = project_type
        self.results_container = results_container

    def prepare_calculation_intervals(self, interval_bg=None, interval_mat=None, interval_incl=None):
        if interval_bg != None:
            condensed_interval = self.combine_all_intervals(interval_set=interval_bg)
        elif interval_mat != None:
            condensed_interval = self.combine_all_intervals(interval_set=interval_mat)
        elif interval_incl != None:
            condensed_interval = self.combine_all_intervals(interval_set=interval_incl)

        return condensed_interval

    def check_if_intervals_isolated(self, intervals_list_01, intervals_list_02=None):
        helper_isolated = []
        helper_overlapped = []
        if intervals_list_02 == None:
            intervals_list_02 = intervals_list_01
        for index_01, interval_01 in enumerate(intervals_list_01):
            for index_02, interval_02 in enumerate(intervals_list_02):
                if interval_02 != interval_01:
                    set_01 = set(range(interval_01[0], interval_01[1]))
                    set_02 = set(range(interval_02[0], interval_02[1]))
                    set_intersection = list(set_01 & set_02)
                    if len(set_intersection) > 0:
                        print("Overlap between:", interval_01, interval_02)
                    else:
                        print("No overlap between:", interval_01, interval_02)

    def combine_all_intervals(self, interval_set):
        condensed_intervals = {}
        if len(interval_set) > 1:
            helper_intervals = []
            for key, item in interval_set.items():
                helper_intervals.append(item["Indices"])
            helper_intervals.sort()
            for index, interval in enumerate(helper_intervals):
                indices_interval = list(range(interval[0], interval[1] + 1))
                if index == 0:
                    all_indices = indices_interval
                all_indices = sorted(np.unique(all_indices + indices_interval))
            all_indices = np.array(all_indices)
            diff_indices = []
            for index, value in enumerate(all_indices):
                if index == 0:
                    diff_indices.append(1)
                else:
                    diff_indices.append(value - all_indices[index - 1])
            indices_jump = [index for index, value in enumerate(diff_indices) if value != 1]
            if len(indices_jump) > 0:
                if len(indices_jump) > 1:
                    for index, value in enumerate(indices_jump):
                        if index == 0:
                            condensed_intervals[index + 1] = [all_indices[0], all_indices[value - 1]]
                            next_start = all_indices[value]
                        else:
                            if index == len(indices_jump) - 1:
                                condensed_intervals[index + 1] = [next_start, all_indices[value - 1]]
                                condensed_intervals[index + 2] = [all_indices[value], all_indices[-1]]
                            else:
                                condensed_intervals[index + 1] = [next_start, all_indices[value - 1]]
                                condensed_intervals[index + 2] = [all_indices[value - 1], all_indices[value]]
                                next_start = all_indices[value]
                else:
                    condensed_intervals[1] = [all_indices[0], all_indices[indices_jump[0] - 1]]
                    condensed_intervals[2] = [all_indices[indices_jump[0]], all_indices[-1]]
            else:
                condensed_intervals[1] = [all_indices[0], all_indices[-1]]
        else:
            for key, item in interval_set.items():
                condensed_intervals[key] = item["Indices"]

        return condensed_intervals

    def get_intensity(self, interval_bg=None, interval_min=None, interval_incl=None, data_key="Data RAW",
                      average_type="arithmetic mean", stack_intervals=False):
        """ Collects the signal intensities from all defined calculation intervals.
        -------
        Parameters
        interval_bg : dict
            Dictionary that contains the defined intervals for the background signal.
        interval_min : dict
            Dictionary that contains the defined intervals for the mineral/matrix signal.
        interval_incl : dict
            Dictionary that contains the defined intervals for the inclusion signal.
        data_key : str
            It specifies if the raw or smoothed data have to be considered.
        average_type : str
            It specifies if the arithmetic mean or the median has to be used.
        stack_intervals : boolean
            It defines if the interval dataset has to be stacked or averaged.
        -------
        Returns
        helper_results : dict
            Dictionary that contains the results.
        -------
        """
        if self.project_type == "MA":
            list_focus = ["BG", "MAT"]
        else:
            list_focus = ["BG", "MAT", "INCL"]

        if data_key == "Data SMOOTHED":
            data_key = "Data IMPROVED"

        helper_results = {}

        for isotope in self.dataframe.keys():
            if self.project_type == "MA":
                helper_results[isotope] = {
                    "uncorrected": {"BG": 0, "MAT": 0, "1 SIGMA BG": 0, "1 SIGMA MAT": 0},
                    "corrected": {"BG": 0, "MAT": 0, "1 SIGMA BG": 0, "1 SIGMA MAT": 0}}
            else:
                helper_results[isotope] = {
                    "uncorrected": {"BG": 0, "MAT": 0, "INCL": 0, "1 SIGMA BG": 0, "1 SIGMA MAT": 0, "1 SIGMA INCL": 0},
                    "corrected": {"BG": 0, "MAT": 0, "INCL": 0, "MIX": 0, "1 SIGMA BG": 0, "1 SIGMA MAT": 0,
                                  "1 SIGMA INCL": 0, "1 SIGMA MIX": 0}}

            for focus in list_focus:
                if focus == "BG":
                    # Background
                    helper_bg = []
                    helper_bg_2 = []
                    helper_bg_sigma = []
                    helper_bg_parts = []
                    for index, interval in interval_bg.items():
                        start_index = interval[0]
                        end_index = interval[1] + 1
                        dataset = self.dataframe[isotope][data_key][start_index:end_index]
                        helper_bg_2.extend(dataset)

                        if stack_intervals == False:
                            if average_type == "arithmetic mean":
                                helper_bg.append(np.mean(dataset))
                            else:
                                helper_bg.append(np.median(dataset))
                            helper_bg_sigma.append(np.std(dataset, ddof=1)/np.sqrt(len(dataset)))
                        else:
                            helper_bg.extend(dataset)

                    n_dataset = len(helper_bg_2)
                    if n_dataset % 2 == 0:
                        n_1st_half = int(len(helper_bg_2)/2)
                    else:
                        n_1st_half = int(round(len(helper_bg_2)/2 + 0.5, 0))

                    if average_type == "arithmetic mean":
                        dataset_1st_half = round(np.mean(helper_bg_2[:n_1st_half]), 4)
                        dataset_2nd_half = round(np.mean(helper_bg_2[n_1st_half:]), 4)
                    else:
                        dataset_1st_half = round(np.median(helper_bg_2[:n_1st_half]), 4)
                        dataset_2nd_half = round(np.median(helper_bg_2[n_1st_half:]), 4)

                    if dataset_2nd_half > 0:
                        parallelism_factor = round(dataset_1st_half/dataset_2nd_half, 4)
                    else:
                        parallelism_factor = np.nan

                    if stack_intervals == True:
                        helper_bg_sigma.append(np.std(helper_bg, ddof=1)/np.sqrt(len(helper_bg)))

                    if average_type == "arithmetic mean":
                        result_bg = np.mean(helper_bg_2)
                        result_bg_1sigma = np.mean(np.std(helper_bg_2, ddof=1)/np.sqrt(len(helper_bg_2)))
                        results_bg_sigma = np.std(helper_bg_2, ddof=1)
                    else:
                        result_bg = np.median(helper_bg_2)
                        result_bg_1sigma = np.median(np.std(helper_bg_2, ddof=1)/np.sqrt(len(helper_bg_2)))
                        results_bg_sigma = np.std(helper_bg_2, ddof=1)

                    helper_results[isotope]["uncorrected"]["BG"] = result_bg
                    helper_results[isotope]["uncorrected"]["1 SIGMA BG"] = result_bg_1sigma
                    helper_results[isotope]["corrected"]["BG"] = result_bg - result_bg
                    helper_results[isotope]["corrected"]["1 SIGMA BG"] = result_bg_1sigma - result_bg_1sigma
                    self.results_container["BG"][isotope] = result_bg
                    self.results_container["BG SIGMA"][isotope] = results_bg_sigma
                    self.results_container["N BG"][isotope] = len(helper_bg_2)
                    self.results_container["Parallelism BG"][isotope] = [dataset_1st_half, dataset_2nd_half]
                elif focus == "MAT":
                    # Mineral/Matrix Signal
                    helper_mat = []
                    helper_mat_sigma = []
                    helper_mat_2 = []
                    for index, interval in interval_min.items():
                        start_index = interval[0]
                        end_index = interval[1] + 1
                        dataset = self.dataframe[isotope][data_key][start_index:end_index]
                        helper_mat_2.extend(dataset)

                        if stack_intervals == False:
                            if average_type == "arithmetic mean":
                                helper_mat.append(np.mean(dataset))
                            else:
                                helper_mat.append(np.median(dataset))
                            helper_mat_sigma.append(np.std(dataset, ddof=1)/np.sqrt(len(dataset)))
                            helper_mat_sigma.append(np.std(dataset, ddof=1)/np.sqrt(len(dataset)))
                        else:
                            helper_mat.extend(dataset)

                    n_dataset = len(helper_mat_2)
                    if n_dataset % 2 == 0:
                        n_1st_half = int(len(helper_mat_2)/2)
                    else:
                        n_1st_half = int(round(len(helper_mat_2)/2 + 0.5, 0))

                    if average_type == "arithmetic mean":
                        dataset_1st_half = round(np.mean(helper_mat_2[:n_1st_half]), 4)
                        dataset_2nd_half = round(np.mean(helper_mat_2[n_1st_half:]), 4)
                    else:
                        dataset_1st_half = round(np.median(helper_mat_2[:n_1st_half]), 4)
                        dataset_2nd_half = round(np.median(helper_mat_2[n_1st_half:]), 4)

                    if dataset_2nd_half > 0:
                        parallelism_factor = round(dataset_1st_half/dataset_2nd_half, 4)
                    else:
                        parallelism_factor = np.nan

                    if stack_intervals == True:
                        helper_mat_sigma.append(np.std(helper_mat, ddof=1)/np.sqrt(len(helper_mat)))

                    if average_type == "arithmetic mean":
                        result_mat = np.mean(helper_mat_2)
                        result_mat_1sigma = np.mean(np.std(helper_mat_2, ddof=1)/np.sqrt(len(helper_mat_2)))
                        results_mat_sigma = np.std(helper_mat_2, ddof=1)
                    else:
                        result_mat = np.median(helper_mat_2)
                        result_mat_1sigma = np.median(np.std(helper_mat_2, ddof=1)/np.sqrt(len(helper_mat_2)))
                        results_mat_sigma = np.std(helper_mat_2, ddof=1)

                    helper_results[isotope]["uncorrected"]["MAT"] = result_mat
                    helper_results[isotope]["uncorrected"]["1 SIGMA MAT"] = result_mat_1sigma
                    helper_results[isotope]["corrected"]["MAT"] = result_mat - result_bg
                    helper_results[isotope]["corrected"]["1 SIGMA MAT"] = result_mat_1sigma - result_bg_1sigma
                    self.results_container["MAT"][isotope] = result_mat
                    self.results_container["1 SIGMA MAT"][isotope] = result_mat_1sigma
                    self.results_container["MAT SIGMA"][isotope] = results_mat_sigma
                    self.results_container["N MAT"][isotope] = len(helper_mat_2)
                    self.results_container["Parallelism MAT"][isotope] = [dataset_1st_half, dataset_2nd_half]
                elif focus == "INCL":
                    # Inclusion Signal
                    helper_incl = []
                    helper_incl_2 = []
                    helper_incl_sigma = []
                    for index, interval in interval_incl.items():
                        start_index = interval[0]
                        end_index = interval[1] + 1
                        dataset = self.dataframe[isotope][data_key][start_index:end_index]
                        helper_incl_2.extend(dataset)

                        if stack_intervals == False:
                            if average_type == "arithmetic mean":
                                helper_incl.append(np.mean(dataset))
                            else:
                                helper_incl.append(np.median(dataset))
                            helper_incl_sigma.append(np.std(dataset, ddof=1)/np.sqrt(len(dataset)))
                        else:
                            helper_incl.extend(dataset)

                    n_dataset = len(helper_incl_2)
                    if n_dataset % 2 == 0:
                        n_1st_half = int(len(helper_incl_2)/2)
                    else:
                        n_1st_half = int(round(len(helper_incl_2)/2 + 0.5, 0))

                    if average_type == "arithmetic mean":
                        dataset_1st_half = round(np.mean(helper_incl_2[:n_1st_half]), 4)
                        dataset_2nd_half = round(np.mean(helper_incl_2[n_1st_half:]), 4)
                    else:
                        dataset_1st_half = round(np.median(helper_incl_2[:n_1st_half]), 4)
                        dataset_2nd_half = round(np.median(helper_incl_2[n_1st_half:]), 4)

                    if dataset_2nd_half > 0:
                        parallelism_factor = round(dataset_1st_half/dataset_2nd_half, 4)
                    else:
                        parallelism_factor = np.nan

                    if stack_intervals == True:
                        helper_incl_sigma.append(np.std(helper_incl, ddof=1)/np.sqrt(len(helper_incl)))

                    if average_type == "arithmetic mean":
                        result_incl = np.mean(helper_incl_2)
                        result_incl_1sigma = np.mean(np.std(helper_incl_2, ddof=1)/np.sqrt(len(helper_incl_2)))
                        results_incl_sigma = np.std(helper_incl_2, ddof=1)
                    else:
                        result_incl = np.median(helper_incl_2)
                        result_incl_1sigma = np.median(np.std(helper_incl_2, ddof=1)/np.sqrt(len(helper_incl_2)))
                        results_incl_sigma = np.std(helper_incl_2, ddof=1)

                    helper_results[isotope]["uncorrected"]["INCL"] = result_incl
                    helper_results[isotope]["uncorrected"]["1 SIGMA INCL"] = result_incl_1sigma
                    helper_results[isotope]["corrected"]["MIX"] = result_incl - result_bg
                    helper_results[isotope]["corrected"]["1 SIGMA MIX"] = result_incl_1sigma - result_bg_1sigma
                    self.results_container["INCL"][isotope] = result_incl
                    self.results_container["1 SIGMA INCL"][isotope] = result_incl_1sigma
                    self.results_container["INCL SIGMA"][isotope] = results_incl_sigma
                    self.results_container["N INCL"][isotope] = len(helper_incl_2)
                    self.results_container["Parallelism INCL"][isotope] = [dataset_1st_half, dataset_2nd_half]

            # for key, item in helper_results[isotope].items():
            #     print(key, item)

        return self.results_container

    def get_intensity_corrected(self, data_container, data_container_ratio=None, isotope_is=None, mode=None,
                                isotope_t=None):
        """ Calculates the background- and partially also matrix-corrected signal intensities.
        -------
        Parameters
        data_container : dict
            Dictionary that contains the previously extracted signal intensities.
        mode : int
            It defines which quantification method has to be used.
        isotope_t : str
            It defines the special isotope that is needed for the quantification.
        -------
        Returns
        helper_results : dict
            Dictionary that contains the results.
        -------
        """
        if self.project_type == "MA":
            list_focus = ["MAT"]
        else:
            list_focus = ["MAT", "INCL"]

        for isotope in data_container["BG"].keys():
            if isotope.isdigit():
                pass
            else:
                intensity_bg_i = data_container["BG"][isotope]
                for focus in list_focus:
                    if focus == "MAT":
                        intensity_sig2_i = data_container[focus][isotope]
                        if None in [intensity_sig2_i, intensity_bg_i]:
                            value_mat_i = None
                        else:
                            value_mat_i = intensity_sig2_i - intensity_bg_i

                        if value_mat_i != None:
                            if value_mat_i > 0:
                                result_mat_i = value_mat_i
                            else:
                                result_mat_i = 0.0
                        else:
                            result_mat_i = np.nan

                        self.results_container["BG"][isotope] = 0.0
                        self.results_container[focus][isotope] = result_mat_i
                    elif focus == "INCL":
                        intensity_sig2_i = data_container["MAT"][isotope]
                        intensity_sig3_i = data_container[focus][isotope]
                        intensity_mix_i = intensity_sig3_i - intensity_bg_i
                        intensity_mat_i = intensity_sig2_i - intensity_bg_i

                        if isotope_t != None:
                            intensity_bg_t = data_container["BG"][isotope_t]
                            intensity_sig2_t = data_container["MAT"][isotope_t]
                            intensity_sig3_t = data_container[focus][isotope_t]
                            intensity_mat_t = intensity_sig2_t - intensity_bg_t

                        if intensity_mix_i > 0:
                            intensity_mix_i = intensity_mix_i
                        else:
                            intensity_mix_i = 0.0

                        if intensity_mat_i > 0:
                            intensity_mat_i = intensity_mat_i
                        else:
                            intensity_mat_i = 0.0

                        if isotope_t != None:
                            if intensity_mat_t > 0:
                                intensity_mat_t = intensity_mat_t
                            else:
                                intensity_mat_t = 0.0

                        if mode == 0:   # Heinrich et al. (2003)
                            intensity_mix_t = intensity_sig3_t - intensity_bg_t
                            value_incl_i = intensity_mix_i - intensity_mix_t*(intensity_mat_i/intensity_mat_t)
                        elif mode == 1: # "SILLS (without R)"
                            intensity_incl_mat_t = intensity_sig3_t - intensity_bg_t
                            intensity_incl_mat_i = (intensity_incl_mat_t/intensity_mat_t)*intensity_mat_i
                            value_incl_i = intensity_mix_i - intensity_incl_mat_i
                        elif mode == 2: # "SILLS (with R)"
                            intensity_incl_mat_t = intensity_sig3_t - intensity_bg_t
                            intensity_incl_mat_i = (intensity_incl_mat_t/intensity_mat_t)*intensity_mat_i
                            if intensity_mat_i > 0:
                                factor_r = intensity_incl_mat_i/intensity_mat_i
                            else:
                                factor_r = 0.0
                            value_incl_i = intensity_mix_i - factor_r*intensity_mat_i
                        elif mode == 3: # "Theory"
                            intensity_incl_mat_t = intensity_sig3_t - intensity_bg_t
                            intensity_incl_mat_i = (intensity_incl_mat_t/intensity_mat_t)*intensity_mat_i
                            value_incl_i = intensity_sig3_i - intensity_bg_i - intensity_incl_mat_i

                        if value_incl_i > 0:
                            result_incl_i = value_incl_i
                        else:
                            result_incl_i = 0.0

                        self.results_container[focus][isotope] = result_incl_i

                        if "MIX" not in self.results_container:
                            self.results_container["MIX"] = {}

                        if isotope not in self.results_container["MIX"]:
                            self.results_container["MIX"][isotope] = intensity_mix_i

                        if "MAT-INCL" not in self.results_container:
                            self.results_container["MAT-INCL"] = {}

                        if isotope not in self.results_container["MAT-INCL"]:
                            try:
                                self.results_container["MAT-INCL"][isotope] = intensity_incl_mat_i
                            except:
                                intensity_incl_mat_i = intensity_sig3_i - result_incl_i - intensity_bg_i
                                self.results_container["MAT-INCL"][isotope] = intensity_incl_mat_i

        return self.results_container

    def get_averaged_intensities(self, data_container, average_type="arithmetic mean"):
        """ Calculates the intensity average (arithmetic mean or median) for all isotopes.
        -------
        Parameters
        data_container : dict
            Dictionary that contains the previously extracted signal intensities.
        average_type : str
            It defines which average has to be used for the quantification.
        -------
        Returns
        helper_results : dict
            Dictionary that contains the results.
        -------
        """
        helper_results = {}
        for filename, item in data_container.items():
            if type(item) == dict:
                for isotope in item["MAT"].keys():
                    if isotope not in helper_results:
                        helper_results[isotope] = []

                    intensity_mat_i = item["MAT"][isotope]
                    helper_results[isotope].append(intensity_mat_i)

        for isotope, item in helper_results.items():
            if isotope.isdigit():
                pass
            else:
                if average_type == "arithmetic mean":
                    result_i = np.mean(item)
                elif average_type == "median":
                    result_i = np.median(item)

                self.results_container[isotope] = result_i

        return self.results_container

    def get_intensity_ratio(self, data_container, dict_is, filename_short=None, datatype=None):
        """ Calculates the intensity ratio for all isotopes.
        -------
        Parameters
        data_container : dict
            Dictionary that contains the previously extracted signal intensities.
        dict_is : dict
            Dictionary that contains information about the internal standard of every file.
        filename_short : str
            Short version of the file of interest (only needed for 'Quick Results' mode).
        datatype : str
            It specifies the data type (only needed for the final 'Results' mode).
        -------
        Returns
        -------
        """
        if self.project_type == "MA":
            list_focus = ["MAT"]
        else:
            list_focus = ["MAT", "INCL"]

        if filename_short != None:
            if filename_short in dict_is["STD"]:
                isotope_is = dict_is["STD"][filename_short]
                for focus in ["MAT"]:
                    value_is = data_container[filename_short][focus][isotope_is]
                    for isotope, value in data_container[filename_short][focus].items():
                        if value_is > 0:
                            result_i = value/value_is
                        else:
                            result_i = np.nan

                        self.results_container[filename_short][focus][isotope] = result_i
            elif filename_short in dict_is["SMPL"]:
                #isotope_is = dict_is["SMPL"][filename_short]
                for focus in list_focus:
                    if focus in dict_is["SMPL"][filename_short]:
                        isotope_is = dict_is["SMPL"][filename_short][focus]
                    else:
                        isotope_is = dict_is["SMPL"][filename_short]

                    value_is = data_container[filename_short][focus][isotope_is]
                    for isotope, value in data_container[filename_short][focus].items():
                        if value_is > 0:
                            result_i = value/value_is
                        else:
                            result_i = np.nan

                        self.results_container[filename_short][focus][isotope] = result_i
            else:
                print("File", filename_short, "was not found!")
        else:
            for filetype in ["STD", "SMPL"]:
                for filename_short, isotope_is in dict_is[filetype].items():
                    if filetype == "STD":
                        list_focus = ["MAT"]
                    else:
                        if self.project_type == "MA":
                            list_focus = ["MAT"]
                        else:
                            list_focus = ["MAT", "INCL"]

                    for focus in list_focus:
                        value_is = data_container[filetype][datatype][filename_short][focus][isotope_is]
                        if value_is != None:
                            for isotope, value in data_container[filetype][datatype][filename_short][focus].items():
                                if value_is > 0:
                                    result_i = value/value_is
                                else:
                                    result_i = np.nan

                                self.results_container[filetype][datatype][filename_short][focus][isotope] = result_i

        return self.results_container

class SensitivityQuantification:
    def __init__(self, dataframe_01, dataframe_02, mode="specific", project_type="MA", results_container=None):
        self.dataframe_01 = dataframe_01
        self.dataframe_02 = dataframe_02
        self.mode = mode
        self.project_type = project_type
        self.results_container = results_container

    def get_normalized_sensitivity(self, filename_short=None, filetype=None, datatype=None, data_sensitivity=None,
                                   dict_is=None):
        """ Calculates the normalized sensitivity for all isotopes.
        -------
        Parameters
        filename_short : str
            Short version of the file of interest (only needed for 'Quick Results' mode).
        filetype : str
            File type (e.g. STD) (only needed for 'Quick Results' mode).
        datatype : str
            Data type (e.g. RAW) (only needed for 'Final Results' mode).
        data_sensitivity : dict
            It contains the analytical sensitivity data.
        dict_is : dict
            It contains information about the internal standard.
        -------
        Returns
        -------
        """
        if filename_short != None:
            if filetype == "STD":
                for isotope, intensity_i in self.dataframe_01["MAT"].items():
                    concentration_i = self.dataframe_02["MAT"][isotope]

                    if intensity_i != None and concentration_i > 0:
                        result_i = intensity_i/concentration_i
                    else:
                        result_i = 0.0

                    self.results_container[filename_short]["MAT"][isotope] = result_i
            else:
                str_is = dict_is[filetype][filename_short]
                intensity_is = self.dataframe_01["MAT"][str_is]
                concentration_is = self.dataframe_02["MAT"][str_is]
                for isotope, intensity_i in self.dataframe_01["MAT"].items():
                    sensitivity_i = data_sensitivity["MAT"][isotope]

                    if concentration_is > 0:
                        result_i = (sensitivity_i*intensity_is)/concentration_is
                    else:
                        result_i = 0.0

                    self.results_container[filename_short]["MAT"][isotope] = result_i
        else:
            for filetype in ["STD", "SMPL"]:
                if filetype == "STD":
                    for filename_short, dataset_01 in self.dataframe_01[filetype][datatype].items():
                        if type(dataset_01) == dict:
                            for isotope, intensity_i in dataset_01["MAT"].items():
                                concentration_i = self.dataframe_02[filetype][datatype][filename_short]["MAT"][isotope]

                                if intensity_i != None and concentration_i > 0:
                                    result_i = intensity_i/concentration_i
                                else:
                                    result_i = 0.0

                                self.results_container[filetype][datatype][filename_short]["MAT"][isotope] = result_i
                else:
                    for filename_short, dataset_01 in self.dataframe_01[filetype][datatype].items():
                        str_is = dict_is[filetype][filename_short]
                        intensity_is = self.dataframe_01[filetype][datatype][filename_short]["MAT"][str_is]
                        concentration_is = self.dataframe_02[filetype][datatype][filename_short]["MAT"][str_is]
                        for isotope, intensity_i in dataset_01["MAT"].items():
                            sensitivity_i = data_sensitivity[filetype][datatype][filename_short]["MAT"][isotope]

                            if concentration_is != None:
                                if concentration_is > 0:
                                    result_i = (sensitivity_i*intensity_is)/concentration_is
                                else:
                                    result_i = np.nan
                            else:
                                result_i = np.nan

                            self.results_container[filetype][datatype][filename_short]["MAT"][isotope] = result_i

        return self.results_container

    def get_analytical_sensitivity(self):
        if self.mode == "specific":
            # Mineral/Matrix Signal

            # Inclusion Signal
            pass
        else:
            # Mineral/Matrix Signal

            # Inclusion Signal
            pass

    def get_relative_sensitivity_factor(self):
        if self.mode == "specific":
            # Mineral/Matrix Signal

            # Inclusion Signal
            pass
        else:
            # Mineral/Matrix Signal

            # Inclusion Signal
            pass

class CompositionQuantification:
    def __init__(self, internal_standard, mode="specific"):
        self.internal_standard = internal_standard
        self.mode = mode

    def get_composition(self):
        if self.mode == "specific":
            # Mineral/Matrix Analysis

            # Fluid Inclusion Analysis

            # Melt Inclusion Analysis
            pass
        else:
            # Mineral/Matrix Analysis

            # Fluid Inclusion Analysis

            # Melt Inclusion Analysis
            pass

    def get_composition_ratio(self):
        if self.mode == "specific":
            # Mineral/Matrix Analysis

            # Fluid Inclusion Analysis

            # Melt Inclusion Analysis
            pass
        else:
            # Mineral/Matrix Analysis

            # Fluid Inclusion Analysis

            # Melt Inclusion Analysis
            pass

    def get_limit_of_detection(self):
        if self.mode == "specific":
            # Mineral/Matrix Analysis

            # Fluid Inclusion Analysis

            # Melt Inclusion Analysis
            pass
        else:
            # Mineral/Matrix Analysis

            # Fluid Inclusion Analysis

            # Melt Inclusion Analysis
            pass

class LinearRegression:
    def __init__(self, x_values, y_values):
        self.x_values = x_values
        self.y_values = y_values

    def calculate_linar_regression(self):
            mean_x = np.mean(self.x_values)
            mean_y = np.mean(self.y_values)
            helper_xy = 0
            helper_x2 = 0
            n = len(self.x_values)

            for index, x_value in enumerate(self.x_values):
                y_value = self.y_values[index]
                helper_xy += x_value*y_value
                helper_x2 += x_value**2

            upper_term = helper_xy - n*mean_x*mean_y
            lower_term = helper_x2 - n*mean_x**2
            b = upper_term/lower_term
            a = mean_y - b*mean_x

            return a, b

## TESTING -------------------------------------------------------------------------------------------------------------


# -*- coding: utf-8 -*-
# Author: Yiping Liu
# Description: This script calculates the average of a list of numbers.
# Version: 1.0
# Last Modified: May 7, 2023

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

# Read the CSV file
df = pd.read_csv("E:\work\data\low-T\error_analysis2.csv")



# Calculate the mean and standard deviation of the "Na" column
mean = np.mean(df["Na"])
std_dev = np.std(df["Na"], ddof=1)

# Calculate the uncertainty of the mean
uncertainty = std_dev / np.sqrt(len(df["Na"]))

# Print the result
print('mean:', mean)
print('std_dev:', std_dev)
print("Uncertainty of the mean of Na column: ", uncertainty)


# Calculate the range of 1 standard deviation
lower_bound = mean - std_dev
upper_bound = mean + std_dev

# Plot the histogram of the "Na" column
plt.hist(df["Na"], bins=40)

# Highlight the range of 1 standard deviation
plt.axvline(x=lower_bound, color='r', linestyle='--')
plt.axvline(x=upper_bound, color='r', linestyle='--')

# Add labels and title
plt.xlabel("Na")
plt.ylabel("Frequency")
plt.title("Histogram of Na column with 1 std dev range")

# Show the plot
plt.show()


# -*- coding: utf-8 -*-
# Author: Yiping Liu
# Description: This script calculates the average of a list of numbers.
# Version: 1.0
# Last Modified: May 7, 2023

import sys
import os
import numpy as np
import matplotlib.pyplot as plt

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
sys.path.append(os.path.dirname(SCRIPT_DIR))

from MCWEP.models_low_t import FluidPitzer

data = {
    't' : [-21.66016784	,0.03317214],
    'Ca': [0.272868977	,0.021785111],
    'K' : [0.073348259	,0.00786989],
    'Li': [0.016466741	,0.001107493],
    'Mg': [0.008402273	,0.001581756],
    'Sr': [0.004591392	,0.000378183],
    'Cs': [0.000260204	,8.67744E-06],
    'Rb': [0.000407099	,5.51968E-05],
    'Mn': [0.001878174	,0.000478762],
    'Zn': [0.001839751	,0.00031582],
    'Pb': [0.000334627	,4.35571E-05],
}

sample_no = 's2_9'

def uncertainty():
    # Set the parameters
    t= data['t' ][0]
    Ca=data['Ca'][0]
    K=data['K'][0]
    Li=data['Li'][0]
    Mg=data['Mg'][0]
    Sr=data['Sr'][0]
    Cs=data['Cs'][0]
    Rb=data['Rb'][0]
    Mn=data['Mn'][0]
    Zn=data['Zn'][0]
    Pb=data['Pb'][0]

    # Set the uncertainties in the parameters
    dt =   data['t'][1]
    dCa = data['Ca'][1]
    dK = data['K'][1]
    dLi = data['Li'][1]
    dMg = data['Mg'][1]
    dSr = data['Sr'][1]
    dCs = data['Cs'][1]
    dRb = data['Rb'][1]
    dMn = data['Mn'][1]
    dZn = data['Zn'][1]
    dPb = data['Pb'][1]

    # Define the number of Monte Carlo simulations to run
    N = 1000
    # Define an array to store the results
    x_values = np.zeros(N)

    # Define arrays to store the values of a_rand and b_rand

    t_rand  =  np.zeros(N)
    Ca_rand = np.zeros(N)
    K_rand = np.zeros(N)
    Li_rand = np.zeros(N)
    Mg_rand = np.zeros(N)
    Sr_rand = np.zeros(N)
    Cs_rand = np.zeros(N)
    Rb_rand = np.zeros(N)
    Mn_rand = np.zeros(N)
    Zn_rand = np.zeros(N)
    Pb_rand = np.zeros(N)

    # Perform the Monte Carlo simulation
    for i in range(N):
        # Generate random values for the parameters based on their uncertainties
        t_rand[i]  = np.random.normal(t ,  dt )
        Ca_rand[i] = np.random.normal(Ca,  dCa)
        K_rand[i]  = np.random.normal(K,   dK)
        Li_rand[i] = np.random.normal(Li,  dLi)
        Mg_rand[i] = np.random.normal(Mg,  dMg)
        Sr_rand[i] = np.random.normal(Sr,  dSr)
        Cs_rand[i] = np.random.normal(Cs,  dCs)
        Rb_rand[i] = np.random.normal(Rb,  dRb)
        Mn_rand[i] = np.random.normal(Mn,  dMn)
        Zn_rand[i] = np.random.normal(Zn,  dZn)
        Pb_rand[i] = np.random.normal(Pb,  dPb)
        # Use the find_x function to find the value of x given the random parameters
        # print('a_rand:',a_rand)

        fluid = FluidPitzer(
            x0=(3.38),
            species={
                'Na+': 1,
                'Ca+2': Ca_rand[i],
                'K+':   K_rand[i],
                'Li+':  Li_rand[i],
                'Mg+2': Mg_rand[i],
                'Sr+2': Sr_rand[i],
                'Cs+':  Cs_rand[i],
                'Rb+':  Rb_rand[i],
                'Mn+2': Mn_rand[i],
                'Zn+2': Zn_rand[i],
                'Pb+2': Pb_rand[i],
            },
            t=t_rand[i],
            solids=['H2O(S)']
        )

        x_values[i] = fluid.find_x()
        print('finished:', i)
    x_mean = np.mean(x_values)
    x_std = np.std(x_values)

    np.savetxt('E:\\work\\data\\low-T\\error_analysis_{}.csv'.format(sample_no), np.column_stack((
        t_rand ,
        Ca_rand,
        K_rand,
        Li_rand,
        Mg_rand,
        Sr_rand,
        Cs_rand,
        Rb_rand,
        Mn_rand,
        Zn_rand,
        Pb_rand,
        x_values
    )), delimiter=',',
               header='t,Ca,K,Li,Mg,Sr,Cs,Rb,Mn,Zn,Pb,Na', comments='')

    print("Mean value of x: ", x_mean)
    print("Standard deviation of x: ", x_std)

uncertainty()



#!/usr/bin/env python
# -*-coding: utf-8 -*-
# ----------------------
# gui.py
# Maximilian Beeskow
# 22.06.2021
# ----------------------
#
## MODULES
from modules import data, plotting
from modules import standard
from modules import sample
from modules import statistics
import numpy as np
import scipy
import matplotlib.pyplot as plt
import tkinter as tk
from tkinter import filedialog, Text
import tkinter.filedialog as fd
import os, re
from matplotlib.backends.backend_tkagg import (FigureCanvasTkAgg, NavigationToolbar2Tk)
from matplotlib.figure import Figure
import time
import sys

# LISTS
list_standards = []
data_standards = []
list_samples = []
data_samples = []

# CLASSES
class Plotting:
    def __init__(self, root, data):
        self.lines = {}
        self.pos = []

        try:
            self.fig.clf()
            self.ax.cla()
            self.canvas.get_tk_widget().pack_forget()
        except AttributeError:
            pass
        try:
            if self.canvas:
                self.canvas.destroy()
            if self.toolbar:
                self.toolbar.destroy()
        except AttributeError:
            pass

        self.canvas = None
        self.toolbar = None
        #
        self.color_background = "#FFFFFF"
        self.root = root
        self.data = data
        self.times = self.data. iloc[:, 0]

        self.frame_main = tk.Frame(self.root, bg=self.color_background)
        self.frame_main.place(relwidth=0.8, relheight=0.8, relx=0.01, rely=0.01)
        #
        self.frame_main_check = tk.Frame(self.frame_main, bg=self.color_background)
        self.frame_main_check.place(relwidth=0.10, relheight=0.92, relx=0.90, rely=0.08)
        self.frame_main_plot = tk.Frame(self.frame_main, bg=self.color_background)
        self.frame_main_plot.place(relwidth=0.895, relheight=1.0, relx=0.0, rely=0.0)

        self.names = list(self.data.columns.values)
        self.names.pop(0)
        self._states = {name: tk.BooleanVar(name=name) for name in self.names}
        self._values = {name: self.data[name] for name in self.names}
        self.values = np.array([[self.data[name] for name in self.names]])
        self.y_max = np.amax(self.values)

        btn_select_all = tk.Button(self.frame_main, text="Select all", height=1, width=10, padx=2, pady=2,
                                   fg="#22252D", bg="#F1F1EB",
                                   command=lambda: self.cb_select_all(states=self._states))
        btn_select_all.place(relx=0.9, rely=0.0)
        btn_deselect_all = tk.Button(self.frame_main, text="Deselect all", height=1, width=10, padx=2, pady=2,
                                     fg="#22252D", bg="#F1F1EB",
                                     command=lambda: self.cb_deselect_all(states=self._states))
        btn_deselect_all.place(relx=0.9, rely=0.04)

        self.fig = Figure(figsize=(1,2), facecolor="#FFFFFF")
        self.ax = self.fig.add_subplot()

        for name, state in self._states.items():
            state.set(True)
            cb = tk.Checkbutton(master=self.frame_main_check, variable=state, text=name,
                                height=1, width=15, anchor=tk.W, bg=self.color_background)
            cb.pack()
            state.trace_add("write", self._value_changed)
            if state.get():
                ln = self.ax.plot(self.times, self._values[name], label=name, visible=True)
                self.lines[name] = ln
        self.ax.grid(True)
        self.ax.set_yscale("log")
        self.ax.set_ylim(bottom=100, top=1.5*self.y_max)
        self.ax.set_axisbelow(True)
        self.ax.set_xlabel("x", labelpad=0.5)
        self.ax.set_ylabel("f(x)", labelpad=0.5)
        self.ax.legend(fontsize="x-small", framealpha=1.0, bbox_to_anchor=(0.125, 0.02), loc=3, borderaxespad=0,
                  bbox_transform=plt.gcf().transFigure, ncol=int(len(self._states)/2))
        #
        self.canvas = FigureCanvasTkAgg(self.fig, master=self.frame_main_plot)
        self.toolbar = NavigationToolbar2Tk(self.canvas, self.frame_main_plot)
        self.toolbar.config(background=self.color_background)
        self.toolbar._message_label.config(background=self.color_background)
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)

        self.canvas.mpl_connect('button_press_event', self.onclick)

    def onclick(self, event):
        self.pos.append([event.xdata,event.ydata])
        print("Cursor position:", self.pos)

    def _value_changed(self, name, _, op):
        if self._states[name].get():
            self.lines[name][0].set_visible(True)
        else:
            self.lines[name][0].set_visible(False)
        self.canvas.draw()

    def cb_select_all(self, states):
        for state in states.values():
            state.set(True)
        state.trace_add("write", self._value_changed)
    #
    def cb_deselect_all(self, states):
        for state in states.values():
            state.set(False)
        state.trace_add("write", self._value_changed)

    def plot_csv(self, parent, listbox, list_data, event):
        t = time.process_time()
        id = listbox.curselection()
        dataset = data.Data(filename=list_data[id[0]])
        df = dataset.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
        Plotting(root=parent, data=df)
        elapsed_time = time.process_time() - t
        print("Execution time:", elapsed_time)

class Plotting2:
    def __init__(self, root, listbox):
        t = time.process_time()
        id = listbox.curselection()
        dataset = data.Data(filename=list_standards[id[0]])
        df = dataset.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)

        self.lines = {}

        try:
            self.fig.clf()
            self.ax.cla()
            self.canvas.get_tk_widget().pack_forget()
        except AttributeError:
            pass
        try:
            if self.canvas:
                self.canvas.destroy()
            if self.toolbar:
                self.toolbar.destroy()
        except AttributeError:
            pass

        self.canvas = None
        self.toolbar = None
        #
        self.color_background = "#FFFFFF"
        self.root = root
        self.data = data
        self.times = self.data. iloc[:, 0]

        self.frame_main = tk.Frame(self.root, bg=self.color_background)
        self.frame_main.place(relwidth=0.8, relheight=0.8, relx=0.01, rely=0.01)
        #
        self.frame_main_check = tk.Frame(self.frame_main, bg=self.color_background)
        self.frame_main_check.place(relwidth=0.10, relheight=0.92, relx=0.90, rely=0.08)
        self.frame_main_plot = tk.Frame(self.frame_main, bg=self.color_background)
        self.frame_main_plot.place(relwidth=0.895, relheight=1.0, relx=0.0, rely=0.0)

        self.names = list(self.data.columns.values)
        self.names.pop(0)
        self._states = {name: tk.BooleanVar(name=name) for name in self.names}
        self._values = {name: self.data[name] for name in self.names}
        self.values = np.array([[self.data[name] for name in self.names]])
        self.y_max = np.amax(self.values)

        btn_select_all = tk.Button(self.frame_main, text="Select all", height=1, width=10, padx=2, pady=2,
                                   fg="#22252D", bg="#F1F1EB",
                                   command=lambda: self.cb_select_all(states=self._states))
        btn_select_all.place(relx=0.9, rely=0.0)
        btn_deselect_all = tk.Button(self.frame_main, text="Deselect all", height=1, width=10, padx=2, pady=2,
                                     fg="#22252D", bg="#F1F1EB",
                                     command=lambda: self.cb_deselect_all(states=self._states))
        btn_deselect_all.place(relx=0.9, rely=0.04)

        self.fig = Figure(figsize=(1,2), facecolor="#FFFFFF")
        self.ax = self.fig.add_subplot()

        for name, state in self._states.items():
            state.set(True)
            cb = tk.Checkbutton(master=self.frame_main_check, variable=state, text=name,
                                height=1, width=15, anchor=tk.W, bg=self.color_background)
            cb.pack()
            state.trace_add("write", self._value_changed)
            if state.get():
                ln = self.ax.plot(self.times, self._values[name], label=name, visible=True)
                self.lines[name] = ln
        self.ax.grid(True)
        self.ax.set_yscale("log")
        self.ax.set_ylim(bottom=100, top=1.5*self.y_max)
        self.ax.set_axisbelow(True)
        self.ax.set_xlabel("x", labelpad=0.5)
        self.ax.set_ylabel("f(x)", labelpad=0.5)
        self.ax.legend(fontsize="x-small", framealpha=1.0, bbox_to_anchor=(0.125, 0.02), loc=3, borderaxespad=0,
                  bbox_transform=plt.gcf().transFigure, ncol=int(len(self._states)/2))
        #
        self.canvas = FigureCanvasTkAgg(self.fig, master=self.frame_main_plot)
        self.toolbar = NavigationToolbar2Tk(self.canvas, self.frame_main_plot)
        self.toolbar.config(background=self.color_background)
        self.toolbar._message_label.config(background=self.color_background)
        self.canvas.get_tk_widget().pack(side=tk.TOP, fill=tk.BOTH, expand=1)

    def create_buttons(self, parent):
        btn_select_all = tk.Button(parent, text="Select all", height=1, width=10, padx=2, pady=2,
                                   fg="#22252D", bg="#F1F1EB",
                                   command=lambda: self.cb_select_all(states=self._states))
        btn_select_all.place(relx=0.9, rely=0.0)
        btn_deselect_all = tk.Button(parent, text="Deselect all", height=1, width=10, padx=2, pady=2,
                                     fg="#22252D", bg="#F1F1EB",
                                     command=lambda: self.cb_deselect_all(states=self._states))
        btn_deselect_all.place(relx=0.9, rely=0.04)

    def _value_changed(self, name, _, op):
        if self._states[name].get():
            self.lines[name][0].set_visible(True)
        else:
            self.lines[name][0].set_visible(False)
        self.canvas.draw()

    def cb_select_all(self, states):
        for state in states.values():
            state.set(True)
        state.trace_add("write", self._value_changed)
    #
    def cb_deselect_all(self, states):
        for state in states.values():
            state.set(False)
        state.trace_add("write", self._value_changed)

    def plot_csv(self, parent, listbox, event):
        t = time.process_time()
        id = listbox.curselection()
        dataset = data.Data(filename=list_standards[id[0]])
        df = dataset.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
        Plotting(root=parent, data=df)
        elapsed_time = time.process_time() - t
        print("Execution time:", elapsed_time)

# FUNCTIONS
def main():

    def newFile():
        print("New File!")
    def openFile():
        name = filedialog.askopenfile()
        print(name)

    def open_csv_standard():
        name = fd.askopenfilenames(parent=root, filetypes=(("csv files", "*.csv"), ("all files", "*.*")))
        for i in name:
            if i not in list_standards:
                list_standards.append(i)
                file_parts = i.split("/")
                #standard_listbox.insert(tk.END, i)
                standard_listbox.insert(tk.END, file_parts[-1])
        return name

    def open_csv_sample():
        name = fd.askopenfilenames(parent=root, filetypes=(("csv files", "*.csv"), ("all files", "*.*")))
        for i in name:
             if i not in list_samples:
                list_samples.append(i)
                file_parts = i.split("/")
                #sample_listbox.insert(tk.END, i)
                sample_listbox.insert(tk.END, file_parts[-1])
        return name

    def delete_csv_standard():
        item = standard_listbox.curselection()
        #list_standards.remove(standard_listbox.get(item))
        list_standards.remove(list_standards[item[0]])
        standard_listbox.delete(tk.ANCHOR)
        return list_standards

    def delete_csv_sample():
        item = sample_listbox.curselection()
        #list_samples.remove(sample_listbox.get(item))
        list_samples.remove(list_samples[item[0]])
        sample_listbox.delete(tk.ANCHOR)
        return list_samples

    def create_list(parent):
        listbox = tk.Listbox(parent, bg="#DFDFDF", width=25, height=17)
        scrollbar_y = tk.Scrollbar(parent, orient="vertical")
        scrollbar_x = tk.Scrollbar(parent, orient="horizontal")
        listbox.config(yscrollcommand = scrollbar_y.set, xscrollcommand = scrollbar_x.set)
        scrollbar_y.config(command = listbox.yview)
        scrollbar_x.config(command = listbox.xview)
        scrollbar_x.pack(side="bottom", fill="x")
        listbox.pack(side="left", fill="both")
        scrollbar_y.pack(side="right", fill="y")
        #scrollbar_x.pack(side="bottom", fill="x")
        return listbox

    def open_csv():
        filename = filedialog.askopenfilename(filetypes=(("csv files", "*.csv"),("all files", "*.*")),
                                              initialdir=os.getcwd())
        return filename

    def saveFile():
        name = filedialog.asksaveasfile()
        #print(name)
    def saveAsFilename():
        name = filedialog.asksaveasfilename()
        #print(name)
    def newWindow():
        toplevel = tk.Toplevel()
        toplevel.title('Another window')
        toplevel.focus_set()

    def add_csv(relx, rely, command):
        btn_add_standard = tk.Button(root, text="Add", height=1, width=6, padx=2, pady=2, fg="#22252D",
                                     bg="#F1F1EB", command=command)
        btn_add_standard.place(relx=relx, rely=rely)
        #btn_add_standard.grid()

    def delete_csv(relx, rely, command):
        btn_delete_standard = tk.Button(root, text="Delete", height=1, width=6, padx=2, pady=2, fg="#22252D",
                                        bg="#F1F1EB", command=command)
        btn_delete_standard.place(relx=relx, rely=rely)

    def analyze_csv(relx, rely, command):
        btn_delete_standard = tk.Button(root, text="Analyze", height=1, width=8, padx=2, pady=2, fg="#22252D",
                                        bg="#F1F1EB", command=command)
        btn_delete_standard.place(relx=relx, rely=rely)

    # def plot_csv_standard(event):
    #     id = standard_listbox.curselection()
    #     dataset = data.Data(filename=list_standards[id[0]])
    #     df = dataset.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
    #     Plotting(root=root, data=df)
    #
    # def plot_csv_sample(event):
    #     id = sample_listbox.curselection()
    #     dataset = data.Data(filename=list_samples[id[0]])
    #     df = dataset.import_data_to_pandas(delimiter=",", skip_header=3, skip_footer=1)
    #     Plotting(root=root, data=df)

    # FRAME
    root = tk.Tk()
    root.title("PySILLS")

    # MENU
    menu = tk.Menu(root)
    root.config(menu=menu)

    filemenu = tk.Menu(menu)
    menu.add_cascade(label="Project", menu=filemenu)
    filemenu.add_command(label="New", command=newFile)
    filemenu.add_command(label="Open", command=openFile)
    filemenu.add_command(label = "Save", command = saveFile)
    filemenu.add_command(label = "Save as", command = saveAsFilename)
    filemenu.add_separator()
    filemenu.add_command(label="Exit", command=root.quit)

    standardmenu = tk.Menu(menu)
    menu.add_cascade(label="Standard", menu=standardmenu)
    standardmenu.add_command(label="Load SRM data", command=open_csv)
    standardmenu.add_command(label="Load SRM measurements", command=open_csv_standard)
    standardmenu.add_separator()
    standardmenu.add_command(label="Settings", command=newWindow)

    samplemenu = tk.Menu(menu)
    menu.add_cascade(label="Sample", menu=samplemenu)
    samplemenu.add_command(label="Load sample measurements", command=open_csv_sample)
    samplemenu.add_separator()
    samplemenu.add_command(label="Settings", command=newWindow)

    resultsmenu = tk.Menu(menu)
    menu.add_cascade(label="Results", menu=resultsmenu)
    resultsmenu.add_command(label="Show sensitivities", command=newWindow)
    resultsmenu.add_command(label="Show concentrations", command=newWindow)

    helpmenu = tk.Menu(menu)
    menu.add_cascade(label="Help", menu=helpmenu)
    helpmenu.add_command(label="Q&A", command=newWindow)

    aboutmenu = tk.Menu(menu)
    menu.add_cascade(label="About", menu=aboutmenu)
    aboutmenu.add_command(label="About PySILLS", command=newWindow)

    canvas = tk.Canvas(root, height=900, width=1600, bg="#2D4D59")
    canvas.pack()

    frame_main = tk.Frame(root, bg="#2D4D59")
    frame_main.place(relwidth=0.8, relheight=0.8, relx=0.01, rely=0.01)
    pysills_logo = tk.PhotoImage(file = "../documentation/images/PySILLS_Logo.png")
    pysills_logo = pysills_logo.subsample(2, 2)
    tk.Label(frame_main, image=pysills_logo, borderwidth=0, highlightthickness=0, padx=0, pady=0).pack(fill="none",
                                                                                                       expand=True)

    # LISTBOX
    frame_standard = tk.Frame(root, bg="#DFDFDF")
    frame_standard.place(relwidth=0.17, relheight=0.32, relx=0.82, rely=0.08)
    standard_listbox = create_list(parent=frame_standard)
    label_standard = tk.Label(root, text ="Standard files", fg="#F1F1EB", bg="#2D4D59")
    label_standard.place(relx = 0.82, rely = 0.01)
    add_csv(relx=0.82, rely=0.04, command=open_csv_standard)
    delete_csv(relx=0.87, rely=0.04, command=delete_csv_standard)
    analyze_csv(relx=0.92, rely=0.04, command=open_csv_standard)
    standard_listbox.bind("<Double-1>", lambda event, parent=root,
                                               listbox=standard_listbox,
                                               list_data=list_standards: Plotting.plot_csv("", parent, listbox,
                                                                                           list_data, event))

    frame_sample = tk.Frame(root, bg="#DFDFDF")
    frame_sample.place(relwidth=0.17, relheight=0.33, relx=0.82, rely=0.48)
    sample_listbox = create_list(parent=frame_sample)
    label_standard = tk.Label(root, text ="Sample files", fg="#F1F1EB", bg="#2D4D59")
    label_standard.place(relx = 0.82, rely = 0.41)
    add_csv(relx=0.82, rely=0.44, command=open_csv_sample)
    delete_csv(relx=0.87, rely=0.44, command=delete_csv_sample)
    analyze_csv(relx=0.92, rely=0.44, command=open_csv_sample)
    sample_listbox.bind("<Double-1>", lambda event, parent=root,
                                               listbox=sample_listbox,
                                               list_data=list_samples: Plotting.plot_csv("", parent, listbox,
                                                                                         list_data, event))

    frame_results = tk.Frame(root, bg="#DFDFDF")
    frame_results.place(relwidth=0.8, relheight=0.17, relx=0.01, rely=0.82)

    # BUTTONS
    btn_bg_sig = tk.Button(root, text="Set background + signal", height=1, width=24, padx=2, pady=2, fg="#22252D",
                           bg="#F1F1EB", command=newWindow)
    btn_bg_sig.place(relx=0.82, rely=0.82)
    set_inclusion = tk.Button(root, text="Set inclusion", height=1, width=24, padx=2, pady=2, fg="#22252D",
                              bg="#F1F1EB", command=newWindow)
    set_inclusion.place(relx=0.82, rely=0.86)
    btn_driftcorr = tk.Button(root, text="Drift correction", height=1, width=24, padx=2, pady=2, fg="#22252D",
                              bg="#F1F1EB", command=newWindow)
    btn_driftcorr.place(relx=0.82, rely=0.90)
    btn_calculation = tk.Button(root, text="Calculation settings", height=1, width=24, padx=2, pady=2, fg="#22252D",
                                bg="#F1F1EB", command=newWindow)
    btn_calculation.place(relx=0.82, rely=0.94)

    Label_middle = tk.Label(root, text ="PySILLS - Version 1.0 - 2021", fg="#F1F1EB", bg="#2D4D59")
    Label_middle.place(relx = 0.83, rely = 0.975)

    root.mainloop()

# PROGRAM
if __name__ == "__main__":
    main()